@@ -27,6 +27,7 @@
 #include <io.h>
 #include <fcntl.h>
 #include <windows.h>
+#define fdopen _fdopen
 #endif
 
 
@@ -57,9 +58,9 @@ static void eprimer3_report (AjPFile outfile, const AjPStr output,
 			     ajint numreturn, ajint begin);
 static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
 				   ajint numreturn, ajint begin);
-static AjPStr eprimer3_tableget(const char * key1, ajint number,
-				const char *key2,
-				const AjPTable table);
+static const AjPStr eprimer3_tableget(const char * key1, ajint number,
+                                      const char *key2,
+                                      const AjPTable table);
 static void eprimer3_write_primer(AjPFile outfile, const char *tag,
 				  const AjPStr pos,
 				  const AjPStr tm, const AjPStr gc,
@@ -186,14 +187,12 @@ int main(int argc, char **argv)
     ajint end;
     FILE* stream;
     AjPStr taskstr  = NULL;
-    AjPStr program  = NULL;
+    const AjPStr program = NULL;
 
-    /* fork/pipe variables */
-#ifndef WIN32
-    pid_t nPid;
-#endif
-    int pipeto[2];	  /* pipe to feed the exec'ed program input */
-    int pipefrom[2];	  /* pipe to get the exec'ed program output */
+    /* pipe variables */
+
+    int *pipeto;	  /* pipe to feed the exec'ed program input */
+    int *pipefrom;	  /* pipe to get the exec'ed program output */
 
     embInit("eprimer3", argc, argv);
 
@@ -209,7 +208,7 @@ int main(int argc, char **argv)
     /* "Sequence" Input Tags */
     sequence        = ajAcdGetSeqall("sequence");
     included_region = ajAcdGetRange("includedregion");
-    target          = ajAcdGetRange("target");
+    target          = ajAcdGetRange("targetregion");
     excluded_region = ajAcdGetRange("excludedregion");
     left_input      = ajAcdGetString("forwardinput");
     right_input     = ajAcdGetString("reverseinput");
@@ -220,10 +219,10 @@ int main(int argc, char **argv)
     max_mispriming      = ajAcdGetFloat("maxmispriming");
     pair_max_mispriming = ajAcdGetFloat("pairmaxmispriming");
     gc_clamp            = ajAcdGetInt("gcclamp");
-    opt_size            = ajAcdGetInt("osize");
+    opt_size            = ajAcdGetInt("optsize");
     min_size            = ajAcdGetInt("minsize");
     max_size            = ajAcdGetInt("maxsize");
-    opt_tm              = ajAcdGetFloat("otm");
+    opt_tm              = ajAcdGetFloat("opttm");
     min_tm              = ajAcdGetFloat("mintm");
     max_tm              = ajAcdGetFloat("maxtm");
     max_diff_tm         = ajAcdGetFloat("maxdifftm");
@@ -237,6 +236,9 @@ int main(int argc, char **argv)
     self_end            = ajAcdGetFloat("selfend");
     max_poly_x          = ajAcdGetInt("maxpolyx");
 
+    AJCNEW0(pipeto,2);
+    AJCNEW0(pipefrom,2);
+    
     /* Sequence Quality */
     /* these are not (yet) implemented */
     /*
@@ -248,11 +250,11 @@ int main(int argc, char **argv)
        */
 
     /* Product details */
-    product_opt_size    = ajAcdGetInt("productosize");
-    product_size_range  = ajAcdGetRange("productsizerange");
-    product_opt_tm      = ajAcdGetFloat("productotm");
-    product_min_tm      = ajAcdGetFloat("productmintm");
-    product_max_tm      = ajAcdGetFloat("productmaxtm");
+    product_opt_size    = ajAcdGetInt("psizeopt");
+    product_size_range  = ajAcdGetRange("prange");
+    product_opt_tm      = ajAcdGetFloat("ptmopt");
+    product_min_tm      = ajAcdGetFloat("ptmmin");
+    product_max_tm      = ajAcdGetFloat("ptmmax");
 
     /* Objective Function Penalty Weights for Primers */
     max_end_stability   = ajAcdGetFloat("maxendstability");
@@ -266,26 +268,26 @@ int main(int argc, char **argv)
     /* these are not (yet) implemented */
 
     /* Internal Oligo "Sequence" Input Tags */
-    internal_oligo_excluded_region = ajAcdGetRange("oligoexcludedregion");
+    internal_oligo_excluded_region = ajAcdGetRange("oexcludedregion");
     internal_oligo_input           = ajAcdGetString("oligoinput");
 
     /* Internal Oligo "Global" Input Tags */
-    internal_oligo_opt_size       = ajAcdGetInt("oligoosize");
-    internal_oligo_min_size       = ajAcdGetInt("oligominsize");
-    internal_oligo_max_size       = ajAcdGetInt("oligomaxsize");
-    internal_oligo_opt_tm         = ajAcdGetFloat("oligootm");
-    internal_oligo_min_tm         = ajAcdGetFloat("oligomintm");
-    internal_oligo_max_tm         = ajAcdGetFloat("oligomaxtm");
-    internal_oligo_opt_gc_percent = ajAcdGetFloat("oligoogcpercent");
-    internal_oligo_min_gc         = ajAcdGetFloat("oligomingc");
-    internal_oligo_max_gc         = ajAcdGetFloat("oligomaxgc");
-    internal_oligo_salt_conc      = ajAcdGetFloat("oligosaltconc");
-    internal_oligo_dna_conc       = ajAcdGetFloat("oligodnaconc");
-    internal_oligo_self_any       = ajAcdGetFloat("oligoselfany");
-    internal_oligo_self_end       = ajAcdGetFloat("oligoselfend");
-    internal_oligo_max_poly_x     = ajAcdGetInt("oligomaxpolyx");
-    internal_oligo_mishyb_library = ajAcdGetInfile("oligomishyblibraryfile");
-    internal_oligo_max_mishyb     = ajAcdGetFloat("oligomaxmishyb");
+    internal_oligo_opt_size       = ajAcdGetInt("osizeopt");
+    internal_oligo_min_size       = ajAcdGetInt("ominsize");
+    internal_oligo_max_size       = ajAcdGetInt("omaxsize");
+    internal_oligo_opt_tm         = ajAcdGetFloat("otmopt");
+    internal_oligo_min_tm         = ajAcdGetFloat("otmmin");
+    internal_oligo_max_tm         = ajAcdGetFloat("otmmax");
+    internal_oligo_opt_gc_percent = ajAcdGetFloat("ogcopt");
+    internal_oligo_min_gc         = ajAcdGetFloat("ogcmin");
+    internal_oligo_max_gc         = ajAcdGetFloat("ogcmax");
+    internal_oligo_salt_conc      = ajAcdGetFloat("osaltconc");
+    internal_oligo_dna_conc       = ajAcdGetFloat("odnaconc");
+    internal_oligo_self_any       = ajAcdGetFloat("oanyself");
+    internal_oligo_self_end       = ajAcdGetFloat("oendself");
+    internal_oligo_max_poly_x     = ajAcdGetInt("opolyxmax");
+    internal_oligo_mishyb_library = ajAcdGetInfile("mishyblibraryfile");
+    internal_oligo_max_mishyb     = ajAcdGetFloat("omishybmax");
     /*
        internal_oligo_min_quality    = ajAcdGetInt("oligominquality");
     */
@@ -298,6 +300,9 @@ int main(int argc, char **argv)
     outfile = ajAcdGetOutfile("outfile");
     
     
+    ajStrRemoveWhite(&left_input);
+    ajStrRemoveWhite(&right_input);
+
     /*
     ** OK - we will now try to do a separate fork-exec for each sequence.
     */
@@ -306,351 +311,194 @@ int main(int argc, char **argv)
 
     while(ajSeqallNext(sequence, &seq))
     {
-#ifndef WIN32
-        /* open the pipes to connect to primer3 */
-        if( pipe( pipeto ) != 0 )
-            ajFatal( "Couldn't open pipe() to" );
+        program = ajAcdGetpathC("primer3_core");
 
-        if( pipe( pipefrom ) != 0 )
-            ajFatal( "Couldn't open pipe() from" );
-    
-    
-        /* fork off the child to run the primer3 executable */
-        nPid = fork();
-        if( nPid < 0 )
-            ajFatal( "Failure of fork()" );
-        else if( nPid == 0 )
+        if(!ajSysExecRedirectC(ajStrGetPtr(program),&pipeto,&pipefrom))
+            ajFatal("eprimer3: Could not exec primer3_core");
+        
+        stream = eprimer3_start_write(pipeto[1]);
+        
+        /* send primer3 Primer "Global" parameters */
+        eprimer3_send_bool(stream, "PRIMER_EXPLAIN_FLAG", explain_flag);
+        eprimer3_send_bool(stream, "PRIMER_FILE_FLAG", file_flag);
+        
+        if(do_hybrid)
         {
-            /*
-            ** CHILD PROCESS
-            ** dup pipe read/write to stdin/stdout
-	    */
-            dup2(pipeto[0],  fileno(stdin));
-            dup2(pipefrom[1], fileno(stdout));
-
-            /* close unnecessary pipe descriptors */
-            close(pipeto[0]);
-            close(pipeto[1]);
-            close(pipefrom[0]);
-            close(pipefrom[1]);
-    
-            program = ajStrNew();
-            ajStrAssignC(&program, "primer3_core");
-    
-            if(ajSysFileWhich(&program))
-            {
-    	        execlp( "primer3_core", "primer3_core", NULL );
-    	        ajDie("There was a problem while executing primer3");
-            }
-            else
-    	        ajDie("The program 'primer3_core' must be on the path.\n"
-		      "It is part of the 'primer3' package, version 1.1,\n"
-		      "available from the Whitehead Institute.\n"
-		      "See: http://primer3.sourceforge.net/");
-    
-            ajStrDel(&program);
-
+            if(!ajStrCmpC(task[0], "1"))
+                ajStrAssignC(&taskstr, "pick_pcr_primers_and_hyb_probe");
+            else if(!ajStrCmpC(task[0], "2"))
+                ajStrAssignC(&taskstr, "pick_left_only");
+            else if(!ajStrCmpC(task[0], "3"))
+                ajStrAssignC(&taskstr, "pick_right_only");
+            else if(!ajStrCmpC(task[0], "4"))
+                ajStrAssignC(&taskstr, "pick_hyb_probe_only");
+        
+            if (!do_primer)
+                ajStrAssignC(&taskstr, "pick_hyb_probe_only");
         }
         else
         {
-            /*
-            ** PARENT PROCESS
-            ** Close unused pipe ends. This is especially important for the
-            ** pipefrom[1] write descriptor, otherwise eprimer3_read will never
-            ** get an EOF.
-            */
-            ajDebug("In PARENT process\n");
-
-            close(pipeto[0]);
-            close(pipefrom[1]);
-
-#else	// WIN32
-	    {
-		HANDLE hChildStdinRd, hChildStdinWr, hChildStdinWrDup, 
-		hChildStdoutRd, hChildStdoutWr, hChildStdoutRdDup, 
-		hSaveStdin, hSaveStdout;
-		BOOL fSuccess;
-		SECURITY_ATTRIBUTES saAttr;
-		saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
-		saAttr.bInheritHandle = TRUE;
-		saAttr.lpSecurityDescriptor = NULL;
-		
-	  // The steps for redirecting child process STDOUT:
-	  //     1. Save current STDOUT, to be restored later. 
-	  //     2. Create anonymous pipe to be STDOUT for child process. 
-	  //     3. Set STDOUT of the parent process to be write handle to 
-	  //        the pipe, so it is inherited by the child process. 
-	  //     4. Create a noninheritable duplicate of the read handle and
-	  //        close the inheritable read handle.
-
-		 hSaveStdout = GetStdHandle(STD_OUTPUT_HANDLE);
-		 if (!CreatePipe(&hChildStdoutRd, &hChildStdoutWr, &saAttr, 0))
-		     ajFatal ( "Couldn't open pipe() from" );
-
-		 SetHandleInformation(hChildStdoutRd,HANDLE_FLAG_INHERIT,0);
-
-		 if (!SetStdHandle(STD_OUTPUT_HANDLE, hChildStdoutWr)) 
-		     ajFatal("Redirecting STDOUT failed");
-		 fSuccess = DuplicateHandle(GetCurrentProcess(),
-                                            hChildStdoutRd,
-					    GetCurrentProcess(),
-                                            &hChildStdoutRdDup , 0,
-					    FALSE,
-					    DUPLICATE_SAME_ACCESS);
-		 if(!fSuccess)
-		     ajFatal("DuplicateHandle failed");
-		 CloseHandle(hChildStdoutRd);
-
-	    // The steps for redirecting child process STDIN: 
-	    //     1.  Save current STDIN, to be restored later. 
-	    //     2.  Create anonymous pipe to be STDIN for child process. 
-	    //     3.  Set STDIN of the parent to be the read handle to the 
-	    //         pipe, so it is inherited by the child process. 
-	    //     4.  Create a noninheritable duplicate of the write handle, 
-	    //         and close the inheritable write handle.
-
-		 hSaveStdin = GetStdHandle(STD_INPUT_HANDLE);
-		 if (!CreatePipe(&hChildStdinRd, &hChildStdinWr, &saAttr, 0)) 
-		     ajFatal ( "Couldn't open pipe() to" );
-		 if (!SetStdHandle(STD_INPUT_HANDLE, hChildStdinRd)) 
-		     ajFatal("Redirecting Stdin failed");
-		 fSuccess = DuplicateHandle(GetCurrentProcess(),
-                                            hChildStdinWr, 
-					    GetCurrentProcess(),
-                                            &hChildStdinWrDup, 0, 
-					    FALSE,         // not inherited 
-					    DUPLICATE_SAME_ACCESS); 
-		  if (!fSuccess) 
-		      ajFatal("DuplicateHandle failed");
-		  CloseHandle(hChildStdinWr);
-
-		  {	// Create the child process
-			PROCESS_INFORMATION piProcInfo;
-			STARTUPINFO siStartInfo;
-			BOOL bFuncRetn = FALSE;
-			char* primer3_core_Dir = getenv("EMBOSS_ROOT");
-			char cmd[MAX_PATH];
-
-			ZeroMemory(&piProcInfo, sizeof(PROCESS_INFORMATION));
-			ZeroMemory(&siStartInfo, sizeof(STARTUPINFO));
-			siStartInfo.cb = sizeof(STARTUPINFO);
-
-			if (primer3_core_Dir == NULL)
-			    ajFatal("Cannot find primer3_core.exe");
-					
-			sprintf(cmd, "%s\\primer3_core.exe", primer3_core_Dir);
-
-			bFuncRetn = CreateProcess(NULL,
-				   cmd,	 // command line 
-				   NULL, // process security attributes 
-				   NULL, // primary thread security attributes 
-				   TRUE, // handles are inherited 
-				   0,    // creation flags 
-				   NULL, // use parent's environment 
-				   NULL, // use parent's current directory 
-				   &siStartInfo,  // STARTUPINFO pointer 
-				   &piProcInfo);  // receives PROCESS_INFORMATION
-			 if (bFuncRetn == 0)
-			     ajFatal("There was a problem executing primer3");
-			 CloseHandle(piProcInfo.hProcess);
-			 CloseHandle(piProcInfo.hThread);
-		  }
-
-	      if (!SetStdHandle(STD_INPUT_HANDLE, hSaveStdin))
-		  ajFatal("Re-redirecting Stdin failed\n");
-	      if (!SetStdHandle(STD_OUTPUT_HANDLE, hSaveStdout))
-		  ajFatal("Re-redirecting Stdout failed\n");
-
-	      CloseHandle(hChildStdoutWr);
-	      pipeto[1] = _open_osfhandle(hChildStdinWrDup, _O_APPEND);
-	      pipefrom[0] = _open_osfhandle(hChildStdoutRdDup, _O_RDONLY);
-#endif	// WIN32
-
-            stream = eprimer3_start_write(pipeto[1]);
-    
-            /* send primer3 Primer "Global" parameters */
-            eprimer3_send_bool(stream, "PRIMER_EXPLAIN_FLAG", explain_flag);
-            eprimer3_send_bool(stream, "PRIMER_FILE_FLAG", file_flag);
-    
-            if(do_hybrid)
-            {
-                if(!ajStrCmpC(task[0], "1"))
-                ajStrAssignC(&taskstr, "pick_pcr_primers_and_hyb_probe");
-                else if(!ajStrCmpC(task[0], "2"))
-                ajStrAssignC(&taskstr, "pick_left_only");
-                else if(!ajStrCmpC(task[0], "3"))
-                ajStrAssignC(&taskstr, "pick_right_only");
-                else if(!ajStrCmpC(task[0], "4"))
-                ajStrAssignC(&taskstr, "pick_hyb_probe_only");
-        
-                if (!do_primer)
-                ajStrAssignC(&taskstr, "pick_hyb_probe_only");
-            }
-            else
-            {
-                if(!ajStrCmpC(task[0], "1"))
+            if(!ajStrCmpC(task[0], "1"))
                 ajStrAssignC(&taskstr, "pick_pcr_primers");
-                else if(!ajStrCmpC(task[0], "2"))
+            else if(!ajStrCmpC(task[0], "2"))
                 ajStrAssignC(&taskstr, "pick_left_only");
-                else if(!ajStrCmpC(task[0], "3"))
+            else if(!ajStrCmpC(task[0], "3"))
                 ajStrAssignC(&taskstr, "pick_right_only");
-                else if(!ajStrCmpC(task[0], "4"))
+            else if(!ajStrCmpC(task[0], "4"))
                 ajStrAssignC(&taskstr, "pick_hyb_probe_only");
-            }
+        }
         
-            eprimer3_send_string(stream, "PRIMER_TASK", taskstr);
-            eprimer3_send_int(stream, "PRIMER_NUM_RETURN", num_return);
-            eprimer3_send_int(stream, "PRIMER_FIRST_BASE_INDEX", 
-                              first_base_index);
-            eprimer3_send_bool(stream, "PRIMER_PICK_ANYWAY", pick_anyway);
+        eprimer3_send_string(stream, "PRIMER_TASK", taskstr);
+        eprimer3_send_int(stream, "PRIMER_NUM_RETURN", num_return);
+        eprimer3_send_int(stream, "PRIMER_FIRST_BASE_INDEX", 
+                          first_base_index);
+        eprimer3_send_bool(stream, "PRIMER_PICK_ANYWAY", pick_anyway);
         
-            /* mispriming library may not have been specified */
-            if(mispriming_library)
+        /* mispriming library may not have been specified */
+        if(mispriming_library)
             eprimer3_send_stringC(stream, "PRIMER_MISPRIMING_LIBRARY",
-                                  ajFileGetNameC(mispriming_library));
+                                  ajFileGetPrintnameC(mispriming_library));
     
-            eprimer3_send_float(stream, "PRIMER_MAX_MISPRIMING", 
-                                max_mispriming);
-            eprimer3_send_float(stream, "PRIMER_PAIR_MAX_MISPRIMING",
-                                pair_max_mispriming);
-            eprimer3_send_int(stream, "PRIMER_GC_CLAMP", gc_clamp);
-            eprimer3_send_int(stream, "PRIMER_OPT_SIZE", opt_size);
-            eprimer3_send_int(stream, "PRIMER_MIN_SIZE", min_size);
-            eprimer3_send_int(stream, "PRIMER_MAX_SIZE", max_size);
-            eprimer3_send_float(stream, "PRIMER_OPT_TM", opt_tm);
-            eprimer3_send_float(stream, "PRIMER_MIN_TM", min_tm);
-            eprimer3_send_float(stream, "PRIMER_MAX_TM", max_tm);
-            eprimer3_send_float(stream, "PRIMER_MAX_DIFF_TM", max_diff_tm);
-            eprimer3_send_float(stream, "PRIMER_OPT_GC_PERCENT", 
-                                opt_gc_percent);
-            eprimer3_send_float(stream, "PRIMER_MIN_GC", min_gc);
-            eprimer3_send_float(stream, "PRIMER_MAX_GC", max_gc);
-            eprimer3_send_float(stream, "PRIMER_SALT_CONC", salt_conc);
-            eprimer3_send_float(stream, "PRIMER_DNA_CONC", dna_conc);
-            eprimer3_send_int(stream, "PRIMER_NUM_NS_ACCEPTED", 
-                              num_ns_accepted);
-            eprimer3_send_float(stream, "PRIMER_SELF_ANY", self_any);
-            eprimer3_send_float(stream, "PRIMER_SELF_END", self_end);
-            eprimer3_send_int(stream, "PRIMER_MAX_POLY_X", max_poly_x);
-            eprimer3_send_int(stream, "PRIMER_PRODUCT_OPT_SIZE", 
-                              product_opt_size);
-            eprimer3_send_range2(stream, "PRIMER_PRODUCT_SIZE_RANGE",
-                                 product_size_range);
-            eprimer3_send_float(stream, "PRIMER_PRODUCT_OPT_TM", 
-                                product_opt_tm);
-            eprimer3_send_float(stream, "PRIMER_PRODUCT_MIN_TM", 
-                                product_min_tm);
-            eprimer3_send_float(stream, "PRIMER_PRODUCT_MAX_TM", 
-                                product_max_tm);
-            eprimer3_send_float(stream, "PRIMER_MAX_END_STABILITY",
-                                max_end_stability);
+        eprimer3_send_float(stream, "PRIMER_MAX_MISPRIMING", 
+                            max_mispriming);
+        eprimer3_send_float(stream, "PRIMER_PAIR_MAX_MISPRIMING",
+                            pair_max_mispriming);
+        eprimer3_send_int(stream, "PRIMER_GC_CLAMP", gc_clamp);
+        eprimer3_send_int(stream, "PRIMER_OPT_SIZE", opt_size);
+        eprimer3_send_int(stream, "PRIMER_MIN_SIZE", min_size);
+        eprimer3_send_int(stream, "PRIMER_MAX_SIZE", max_size);
+        eprimer3_send_float(stream, "PRIMER_OPT_TM", opt_tm);
+        eprimer3_send_float(stream, "PRIMER_MIN_TM", min_tm);
+        eprimer3_send_float(stream, "PRIMER_MAX_TM", max_tm);
+        eprimer3_send_float(stream, "PRIMER_MAX_DIFF_TM", max_diff_tm);
+        eprimer3_send_float(stream, "PRIMER_OPT_GC_PERCENT", 
+                            opt_gc_percent);
+        eprimer3_send_float(stream, "PRIMER_MIN_GC", min_gc);
+        eprimer3_send_float(stream, "PRIMER_MAX_GC", max_gc);
+        eprimer3_send_float(stream, "PRIMER_SALT_CONC", salt_conc);
+        eprimer3_send_float(stream, "PRIMER_DNA_CONC", dna_conc);
+        eprimer3_send_int(stream, "PRIMER_NUM_NS_ACCEPTED", 
+                          num_ns_accepted);
+        eprimer3_send_float(stream, "PRIMER_SELF_ANY", self_any);
+        eprimer3_send_float(stream, "PRIMER_SELF_END", self_end);
+        eprimer3_send_int(stream, "PRIMER_MAX_POLY_X", max_poly_x);
+        eprimer3_send_int(stream, "PRIMER_PRODUCT_OPT_SIZE", 
+                          product_opt_size);
+        eprimer3_send_range2(stream, "PRIMER_PRODUCT_SIZE_RANGE",
+                             product_size_range);
+        eprimer3_send_float(stream, "PRIMER_PRODUCT_OPT_TM", 
+                            product_opt_tm);
+        eprimer3_send_float(stream, "PRIMER_PRODUCT_MIN_TM", 
+                            product_min_tm);
+        eprimer3_send_float(stream, "PRIMER_PRODUCT_MAX_TM", 
+                            product_max_tm);
+        eprimer3_send_float(stream, "PRIMER_MAX_END_STABILITY",
+                            max_end_stability);
     
-            /* send primer3 Internal Oligo "Global" parameters */
-            eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_OPT_SIZE",
-                    internal_oligo_opt_size);
-            eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MIN_SIZE",
-                    internal_oligo_min_size);
-            eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MAX_SIZE",
-                    internal_oligo_max_size);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_OPT_TM",
-                    internal_oligo_opt_tm);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MIN_TM",
-                    internal_oligo_min_tm);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_TM",
-                    internal_oligo_max_tm);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_OPT_GC_PERCENT",
-                    internal_oligo_opt_gc_percent);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MIN_GC",
-                    internal_oligo_min_gc);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_GC",
-                    internal_oligo_max_gc);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SALT_CONC",
-                    internal_oligo_salt_conc);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_DNA_CONC",
-                    internal_oligo_dna_conc);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SELF_ANY",
-                    internal_oligo_self_any);
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SELF_END",
-                    internal_oligo_self_end);
-            eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MAX_POLY_X",
-                    internal_oligo_max_poly_x);
+        /* send primer3 Internal Oligo "Global" parameters */
+        eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_OPT_SIZE",
+                          internal_oligo_opt_size);
+        eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MIN_SIZE",
+                          internal_oligo_min_size);
+        eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MAX_SIZE",
+                          internal_oligo_max_size);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_OPT_TM",
+                            internal_oligo_opt_tm);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MIN_TM",
+                            internal_oligo_min_tm);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_TM",
+                            internal_oligo_max_tm);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_OPT_GC_PERCENT",
+                            internal_oligo_opt_gc_percent);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MIN_GC",
+                            internal_oligo_min_gc);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_GC",
+                            internal_oligo_max_gc);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SALT_CONC",
+                            internal_oligo_salt_conc);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_DNA_CONC",
+                            internal_oligo_dna_conc);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SELF_ANY",
+                            internal_oligo_self_any);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_SELF_END",
+                            internal_oligo_self_end);
+        eprimer3_send_int(stream, "PRIMER_INTERNAL_OLIGO_MAX_POLY_X",
+                          internal_oligo_max_poly_x);
 
-            /* 
-            ** internal oligo mishybridising library may not have been
-            ** specified 
-            */
-            if(internal_oligo_mishyb_library)
-                eprimer3_send_stringC(stream,
-                    "PRIMER_INTERNAL_OLIGO_MISHYB_LIBRARY",
-                    ajFileGetNameC(internal_oligo_mishyb_library));
+        /* 
+        ** internal oligo mishybridising library may not have been
+        ** specified 
+        */
+        if(internal_oligo_mishyb_library)
+            eprimer3_send_stringC(stream,
+                          "PRIMER_INTERNAL_OLIGO_MISHYB_LIBRARY",
+                          ajFileGetPrintnameC(internal_oligo_mishyb_library));
 
-            eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_MISHYB",
-                    internal_oligo_max_mishyb);
+        eprimer3_send_float(stream, "PRIMER_INTERNAL_OLIGO_MAX_MISHYB",
+                            internal_oligo_max_mishyb);
         
         
-            /* 
-            ** Start sequence-specific stuff 
-            */
+        /* 
+        ** Start sequence-specific stuff 
+        */
 
-            begin = ajSeqallGetseqBegin(sequence) - 1;
-            end   = ajSeqallGetseqEnd(sequence) - 1;
+        begin = ajSeqallGetseqBegin(sequence) - 1;
+        end   = ajSeqallGetseqEnd(sequence) - 1;
 
-            strand = ajSeqGetSeqCopyS(seq);
+        strand = ajSeqGetSeqCopyS(seq);
 
-            ajStrFmtUpper(&strand);
-            ajStrAssignSubC(&substr,ajStrGetPtr(strand), begin, end);
+        ajStrFmtUpper(&strand);
+        ajStrAssignSubC(&substr,ajStrGetPtr(strand), begin, end);
 
-            /* send flags to turn on using optimal product size */
-            eprimer3_send_float(stream, "PRIMER_PAIR_WT_PRODUCT_SIZE_GT",
-                                0.05);
-            eprimer3_send_float(stream, "PRIMER_PAIR_WT_PRODUCT_SIZE_LT",
-                                0.05);
+        /* send flags to turn on using optimal product size */
+        eprimer3_send_float(stream, "PRIMER_PAIR_WT_PRODUCT_SIZE_GT",
+                            (float)0.05);
+        eprimer3_send_float(stream, "PRIMER_PAIR_WT_PRODUCT_SIZE_LT",
+                            (float)0.05);
 
-            /* send primer3 Primer "Sequence" parameters */
-            eprimer3_send_string(stream, "SEQUENCE", substr);
+        /* send primer3 Primer "Sequence" parameters */
+        eprimer3_send_string(stream, "SEQUENCE", substr);
 
-            /* if no ID name, use the USA */
-            if(ajStrMatchC(ajSeqGetNameS(seq),""))
-                eprimer3_send_string(stream, "PRIMER_SEQUENCE_ID",
-                                     ajSeqGetUsaS(seq));
-            else
-                eprimer3_send_string(stream, "PRIMER_SEQUENCE_ID",
-                                     ajSeqGetNameS(seq));
+        /* if no ID name, use the USA */
+        if(ajStrMatchC(ajSeqGetNameS(seq),""))
+            eprimer3_send_string(stream, "PRIMER_SEQUENCE_ID",
+                                 ajSeqGetUsaS(seq));
+        else
+            eprimer3_send_string(stream, "PRIMER_SEQUENCE_ID",
+                                 ajSeqGetNameS(seq));
 
-            eprimer3_send_range(stream, "INCLUDED_REGION", included_region,
-                                begin);
-            eprimer3_send_range(stream, "TARGET", target, begin);
-            eprimer3_send_range(stream, "EXCLUDED_REGION", excluded_region,
-                                begin);
-            eprimer3_send_string(stream, "PRIMER_LEFT_INPUT", left_input);
-            eprimer3_send_string(stream, "PRIMER_RIGHT_INPUT", right_input);
+        eprimer3_send_range(stream, "INCLUDED_REGION", included_region,
+                            begin);
+        eprimer3_send_range(stream, "TARGET", target, begin);
+        eprimer3_send_range(stream, "EXCLUDED_REGION", excluded_region,
+                            begin);
+        eprimer3_send_string(stream, "PRIMER_LEFT_INPUT", left_input);
+        eprimer3_send_string(stream, "PRIMER_RIGHT_INPUT", right_input);
 
-            /* send primer3 Internal Oligo "Sequence" parameters */
-            eprimer3_send_range(stream,
-                                "PRIMER_INTERNAL_OLIGO_EXCLUDED_REGION",
-                                internal_oligo_excluded_region, begin);
-            eprimer3_send_string(stream, "PRIMER_INTERNAL_OLIGO_INPUT",
-                                 internal_oligo_input);
+        /* send primer3 Internal Oligo "Sequence" parameters */
+        eprimer3_send_range(stream,
+                            "PRIMER_INTERNAL_OLIGO_EXCLUDED_REGION",
+                            internal_oligo_excluded_region, begin);
+        eprimer3_send_string(stream, "PRIMER_INTERNAL_OLIGO_INPUT",
+                             internal_oligo_input);
 
 
-            /* end the primer3 input sequence record with a '=' */
-            eprimer3_send_end(stream);
-            /* and close the ouput pipe stream */
-            eprimer3_end_write(stream);
+        /* end the primer3 input sequence record with a '=' */
+        eprimer3_send_end(stream);
+        /* and close the ouput pipe stream */
+        eprimer3_end_write(stream);
     
-            /* read the primer3 output */
-            eprimer3_read(pipefrom[0], &result);
+        /* read the primer3 output */
+        eprimer3_read(pipefrom[0], &result);
             
-            eprimer3_report(outfile, result, num_return, begin);
+        eprimer3_report(outfile, result, num_return, begin);
     
-            ajStrSetClear(&result);
+        ajStrSetClear(&result);
 
 #ifndef WIN32
-            close(pipeto[1]);
-            close(pipefrom[0]);
+        close(pipeto[1]);
+        close(pipefrom[0]);
 #endif            
-        }	/* end of parent/child fork */
-
     }	/* end of sequence loop */
 
 
@@ -675,6 +523,9 @@ int main(int argc, char **argv)
     ajStrDel(&right_input);
     ajStrDel(&internal_oligo_input);
 
+    AJFREE(pipeto);
+    AJFREE(pipefrom);
+    
     ajFileClose(&mispriming_library);
 
     embExit();
@@ -762,15 +613,15 @@ static void eprimer3_send_range(FILE * stream, const char * tag,
 
     str = ajStrNew();
 
-    if(ajRangeNumber(value))
+    if(ajRangeGetSize(value))
     {
         ajFmtPrintS(&str, "%s=", tag);
         eprimer3_write(str, stream);
         ajStrSetClear(&str);
 
-        for(n=0; n < ajRangeNumber(value); n++)
+        for(n=0; n < ajRangeGetSize(value); n++)
         {
-            ajRangeValues(value, n, &start, &end);
+            ajRangeElementGetValues(value, n, &start, &end);
             start -= begin;
             end   -= begin;
             ajFmtPrintS(&str, "%d,%d ", start, end-start+1);
@@ -811,15 +662,15 @@ static void eprimer3_send_range2(FILE * stream, const char * tag,
 
     str=ajStrNew();
 
-    if(ajRangeNumber(value))
+    if(ajRangeGetSize(value))
     {
         ajFmtPrintS(&str, "%s=", tag);
         eprimer3_write(str, stream);
         ajStrSetClear(&str);
 
-        for(n=0; n < ajRangeNumber(value); n++)
+        for(n=0; n < ajRangeGetSize(value); n++)
         {
-            ajRangeValues(value, n, &start, &end);
+            ajRangeElementGetValues(value, n, &start, &end);
             ajFmtPrintS(&str, "%d-%d ", start, end);
             eprimer3_write(str, stream);
             ajStrSetClear(&str);
@@ -1090,7 +941,7 @@ static void eprimer3_report(AjPFile outfile, const AjPStr output,
             if(ajStrCmpLenC(line, "PRIMER_SEQUENCE_ID=", 19) == 0)
             {
                 gotsequenceid = AJTRUE;
-                table = ajTablestrNewLen(TABLEGUESS);
+                table = ajTablestrNew(TABLEGUESS);
 
             }
             else
@@ -1160,16 +1011,16 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
                                    ajint numreturn, ajint begin)
 {
     AjPStr key   = NULL;
-    AjPStr error = NULL;		/*  value from table - do not delete */
     AjPStr explain    = NULL;
     AjPStr explainstr = NULL;
-    AjPStr seqid      = NULL;
+    const AjPStr error = NULL;		/*  value from table - do not delete */
+    const AjPStr seqid      = NULL;
     ajint i;
-    AjPStr size = NULL;
-    AjPStr seq  = NULL;
-    AjPStr pos  = NULL;
-    AjPStr tm   = NULL;
-    AjPStr gc   = NULL;
+    const AjPStr size = NULL;
+    const AjPStr seq  = NULL;
+    const AjPStr pos  = NULL;
+    const AjPStr tm   = NULL;
+    const AjPStr gc   = NULL;
 
      /*
      ** Typical primer3 output looks like:
@@ -1205,24 +1056,24 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
   
     /* check for errors */
     ajStrAssignC(&key, "PRIMER_ERROR");
-    error =(AjPStr)ajTableFetch(table,(const void*)key);
+    error = ajTableFetchS(table, key);
     if(error != NULL)
         ajErr("%S", error);
 
     ajStrAssignC(&key, "PRIMER_WARNING");
-    error = (AjPStr)ajTableFetch(table,(const void*)key);
+    error = ajTableFetchS(table, key);
     if(error != NULL)
         ajWarn("%S", error);
 
   
     /* get the sequence id */
     ajStrAssignC(&key, "PRIMER_SEQUENCE_ID");
-    seqid = (AjPStr)ajTableFetch(table,(const void*)key);
+    seqid = ajTableFetchS(table, key);
     ajFmtPrintF(outfile, "\n# EPRIMER3 RESULTS FOR %S\n\n", seqid);
   
     /* get information on the analysis */
     ajStrAssignC(&key, "PRIMER_LEFT_EXPLAIN");
-    explain = (AjPStr)ajTableFetch(table,(const void*)key);
+    explain = ajTableFetchmodS(table, key);
 
     if(explain != NULL)
     {
@@ -1232,7 +1083,7 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
                     explainstr);
     }
     ajStrAssignC(&key, "PRIMER_RIGHT_EXPLAIN");
-    explain = (AjPStr)ajTableFetch(table,(const void*)key);
+    explain = ajTableFetchmodS(table, key);
 
     if(explain != NULL)
     {
@@ -1242,7 +1093,7 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
                     explainstr);
     }
     ajStrAssignC(&key, "PRIMER_PAIR_EXPLAIN");
-    explain = (AjPStr)ajTableFetch(table,(const void*)key);
+    explain = ajTableFetchmodS(table, key);
 
     if(explain != NULL)
     {
@@ -1252,7 +1103,7 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
                     explainstr);
     }
     ajStrAssignC(&key, "PRIMER_INTERNAL_OLIGO_EXPLAIN");
-    explain = (AjPStr)ajTableFetch(table,(const void*)key);
+    explain = ajTableFetchmodS(table, key);
 
     if(explain != NULL)
     {
@@ -1326,17 +1177,17 @@ static void eprimer3_output_report(AjPFile outfile, const AjPTable table,
 ** @param [r] key2 [const char *] Second half of table key base string
 **                                (minus '_')
 ** @param [r] table [const AjPTable] Table of tag/value result pairs
-** @return [AjPStr] Table value
+** @return [const AjPStr] Table value
 **
 ******************************************************************************/
 
-static AjPStr eprimer3_tableget(const char *key1, ajint number,
+static const AjPStr eprimer3_tableget(const char *key1, ajint number,
 				const char *key2,
                                 const AjPTable table)
 {
     AjPStr fullkey = NULL;
     AjPStr keynum  = NULL;
-    AjPStr value   = NULL;
+    const AjPStr value   = NULL;
 
     ajStrAssignC(&fullkey, key1);
 
@@ -1352,7 +1203,7 @@ static AjPStr eprimer3_tableget(const char *key1, ajint number,
 
     ajStrAppendC(&fullkey, key2);
     ajDebug("Constructed key=%S\n", fullkey);
-    value =(AjPStr)ajTableFetch(table,(const void*)fullkey);
+    value = ajTableFetchS(table, fullkey);
 
 
     ajStrDel(&fullkey);
@@ -1392,7 +1243,7 @@ static void eprimer3_write_primer(AjPFile outfile, const char *tag,
     float gcfloat;
     AjPStr start = NULL;
     AjPStr lenstr = NULL;
-    ajint comma;
+    ajlong comma;
 
     if(ajStrGetLen(pos))
     {
