@@ -25,7 +25,7 @@
 
 
 #define ENZDATA "REBASE/embossre.enz"
-#define EQUDATA "embossre.equ"
+#define EQUDATA "REBASE/embossre.equ"
 #define EQUGUESS 3500     /* Estimate of number of equivalent names */
 
 #define TABLEGUESS 200
@@ -57,6 +57,8 @@ static AjBool remap_Ambiguous(const AjPStr str);
 static void remap_GetFrames(AjPStr const *framelist, AjBool *frames);
 
 
+
+
 /* @datastatic PValue *********************************************************
 **
 ** structure for counts and isoschizomers of a restriction enzyme hit
@@ -135,8 +137,10 @@ int main(int argc, char **argv)
     AjBool plasmid;
     AjBool commercial;
     AjBool limit;
+    AjBool methyl;
     AjPFile enzfile  = NULL;
     AjPFile equfile  = NULL;
+    AjPFile methfile = NULL;
     AjPTable retable = NULL;
     ajint hits;
     AjPList restrictlist = NULL;
@@ -176,7 +180,9 @@ int main(int argc, char **argv)
     commercial = ajAcdGetBoolean("commercial");
     limit      = ajAcdGetBoolean("limit");
     enzymes    = ajAcdGetString("enzymes");
-
+    methfile   = ajAcdGetDatafile("mfile");
+    methyl     = ajAcdGetBoolean("methylation");
+    
     if(!blunt  && !sticky)
 	ajFatal("Blunt/Sticky end cutters shouldn't both be disabled.");
 
@@ -232,7 +238,7 @@ int main(int argc, char **argv)
 	 */
 	if(single)
 	    maxcuts=mincuts=1;
-	retable = ajTablestrNewLen(EQUGUESS);
+	retable = ajTablestrNew(EQUGUESS);
 	enzfile = ajDatafileNewInNameC(ENZDATA);
 	if(!enzfile)
 	    ajFatal("Cannot locate enzyme file. Run REBASEEXTRACT");
@@ -249,9 +255,10 @@ int main(int argc, char **argv)
 	ajFileSeek(enzfile, 0L, 0);
 	restrictlist = ajListNew();
 	/* search for hits, but don't use mincuts and maxcuts criteria yet */
-	hits = embPatRestrictMatch(seq, begin+1, end+1, enzfile, enzymes,
-				   sitelen,plasmid, ambiguity, default_mincuts,
-				   default_maxcuts, blunt, sticky, commercial,
+	hits = embPatRestrictMatch(seq, begin+1, end+1, enzfile, methfile,
+                                   enzymes, sitelen,plasmid, ambiguity,
+                                   default_mincuts, default_maxcuts, blunt,
+                                   sticky, commercial, methyl,
 				   restrictlist);
 
 	ajDebug("Remap found %d hits\n", hits);
@@ -267,6 +274,7 @@ int main(int argc, char **argv)
 
 
 	ajFileClose(&enzfile);
+	ajFileClose(&methfile);
 
 
 	/*
@@ -274,7 +282,7 @@ int main(int argc, char **argv)
 	** criteria, but save them in hittable for printing out later.
 	** Keep a count of how many hits each enzyme gets in hittable.
 	*/
-        hittable = ajTablestrNewCaseLen(TABLEGUESS);
+        hittable = ajTablestrNewCase(TABLEGUESS);
 	remap_RemoveMinMax(restrictlist, hittable, mincuts, maxcuts);
 
 
@@ -403,7 +411,7 @@ static void remap_DelTable(AjPTable * table)
 
     if(ajTableGetLength(*table))
     {
-      ajTableToarray(*table, &keyarray, &valarray);
+      ajTableToarrayKeysValues(*table, &keyarray, &valarray);
       for(i = 0; keyarray[i]; i++)
       {
           value = (PValue) valarray[i];
@@ -459,7 +467,7 @@ static void remap_RemoveMinMax(AjPList restrictlist,
 	    ajStrAssignS(&key, m->cod);
 
 	    /* increment the count of key */
-	    value = (PValue) ajTableFetch(hittable, (const void *)key);
+	    value = (PValue) ajTableFetchmodS(hittable, key);
 	    if(value == NULL)
 	    {
 		AJNEW0(value);
@@ -480,7 +488,7 @@ static void remap_RemoveMinMax(AjPList restrictlist,
 	miter = ajListIterNew(restrictlist);
 	while((m = ajListIterGet(miter)) != NULL)
 	{
-	    value = (PValue) ajTableFetch(hittable, (const void *)(m->cod));
+	    value = (PValue) ajTableFetchmodS(hittable, (m->cod));
             if(value->count < mincuts || value->count > maxcuts)
 	    {
             	ajListIterRemove(miter);
@@ -516,9 +524,8 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 			  AjBool isos,
 			  AjBool html, ajint mincuts, ajint maxcuts)
 {
-    PValue value;
+    const PValue value;
     void **keyarray = NULL;		/* array for table */
-    void **valarray = NULL;		/* array for table */
     ajint i;
 
     /* print title */
@@ -535,8 +542,8 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 
     if(ajTableGetLength(hittable))
     {
-        ajTableToarray(hittable, &keyarray, &valarray);
-        qsort(keyarray, ajTableGetLength(hittable), sizeof (*keyarray),
+        ajTableToarrayKeys(hittable, &keyarray);
+        qsort(keyarray, (size_t) ajTableGetLength(hittable), sizeof (*keyarray),
 	      ajStrVcmp);
 
 	/* enzymes that cut the required number of times */
@@ -545,7 +552,7 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 
 	for(i = 0; keyarray[i]; i++)
 	{
-	    value = ajTableFetch(hittable,keyarray[i]);
+	    value = ajTableFetchS(hittable,keyarray[i]);
 	    if(value->count >= mincuts && value->count <= maxcuts)
 	    ajFmtPrintF(outfile, "%10S\t    %d\t%S\n",
 		    (AjPStr) keyarray[i], value->count,
@@ -582,7 +589,7 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 
 	for(i = 0; keyarray[i]; i++)
 	{
-	    value = ajTableFetch(hittable,keyarray[i]);
+	    value = ajTableFetchS(hittable,keyarray[i]);
 	    if(value->count < mincuts)
 	    ajFmtPrintF(outfile, "%10S\t    %d\t%S\n",
 			    (AjPStr) keyarray[i], value->count,
@@ -618,7 +625,7 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 
 	for(i = 0; keyarray[i]; i++)
 	{
-	    value = (PValue) valarray[i];
+	    value = ajTableFetchS(hittable,keyarray[i]);
 	    if(value->count > maxcuts)
 	    ajFmtPrintF(outfile, "%10S\t    %d\t%S\n",
 			    (AjPStr) keyarray[i], value->count,
@@ -631,7 +638,6 @@ static void remap_CutList(AjPFile outfile, const AjPTable hittable,
 	    ajFmtPrintF(outfile, "</PRE>\n");
 
 	AJFREE(keyarray);
-	AJFREE(valarray);
     }
 
     return;
@@ -716,7 +722,7 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
     nocutlist = ajListstrNew();
 
 
-    ajTableToarray(hittable, &keyarray, &valarray);
+    ajTableToarrayKeysValues(hittable, &keyarray, &valarray);
     for(i = 0; keyarray[i]; i++)
     {
         value = (PValue) valarray[i];
@@ -773,8 +779,11 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
     /* push all enzyme names without the required criteria onto nocutlist */
 
     enz = embPatRestrictNew();
-    while(embPatRestrictReadEntry(enz, enzfile))
+    while(!ajFileIsEof(enzfile))
     {
+        if(!embPatRestrictReadEntry(enz, enzfile))
+	    continue;
+
          /* 
 	 ** If user entered explicit enzyme list, then check to see if
 	 ** this is one of that explicit list 
@@ -883,7 +892,7 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
         remap_RenamePreferred(nocutlist, retable, newlist);
         ajListstrFreeData(&nocutlist);
         nocutlist = newlist;
-        ajListSortUnique(nocutlist, remap_cmpcase, remap_strdel);
+        ajListSortUnique(nocutlist, &remap_cmpcase, &remap_strdel);
     }
 
 
@@ -902,8 +911,8 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
      **  nocutlist item is deleted.
      */
 
-    ajListSort(nocutlist, remap_cmpcase);
-    ajListSort(cutlist, remap_cmpcase);
+    ajListSort(nocutlist, &remap_cmpcase);
+    ajListSort(cutlist, &remap_cmpcase);
 
     citer = ajListIterNewread(cutlist);
     niter = ajListIterNew(nocutlist);
@@ -965,7 +974,7 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
     if(html)
 	ajFmtPrintF(outfile, "<PRE>");
 
-    /*  ajListSort(nocutlist, ajStrVcmp);*/
+    /*  ajListSort(nocutlist, &ajStrVcmp);*/
     niter = ajListIterNewread(nocutlist);
     i = 0;
     while((nocutname = (AjPStr)ajListIterGet(niter)) != NULL)
@@ -997,7 +1006,7 @@ static void remap_NoCutList(AjPFile outfile, const AjPTable hittable,
         ajFmtPrintF(outfile, "<H2>");
     ajFmtPrintF(outfile,
 		"\n\n# No. of cutting enzymes which do not match the\n"
-		"# SITELEN, BLUNT, STICKY, COMMERCIAL, AMBIGUOUS citeria\n\n");
+		"# SITELEN, BLUNT, STICKY, COMMERCIAL, AMBIGUOUS criteria\n\n");
     if(html)
 	ajFmtPrintF(outfile, "</H2>\n");
     ajFmtPrintF(outfile, "%d\n", rejected_count);
@@ -1186,7 +1195,7 @@ static void remap_RenamePreferred(const AjPList list, const AjPTable table,
 {
     AjIList iter = NULL;
     AjPStr key   = NULL;
-    AjPStr value = NULL;
+    const AjPStr value = NULL;
     AjPStr name  = NULL;
 
     iter = ajListIterNewread(list);
@@ -1197,7 +1206,7 @@ static void remap_RenamePreferred(const AjPList list, const AjPTable table,
         ** If a key-value entry found, write the new value to the new list
         ** else write the old key name to the new list
         */
-        value = ajTableFetch(table, key);
+        value = ajTableFetchS(table, key);
         name = ajStrNew();
         if(value)
 	{
@@ -1241,7 +1250,7 @@ static void remap_RestrictPreferred(const AjPList l, const AjPTable t)
 {
     AjIList iter   = NULL;
     EmbPMatMatch m = NULL;
-    AjPStr value   = NULL;
+    const AjPStr value   = NULL;
     AjPStr newiso  = NULL;
     AjBool found;		/* name found in isoschizomer list */
         	    
@@ -1257,7 +1266,7 @@ static void remap_RestrictPreferred(const AjPList l, const AjPTable t)
 	found = ajFalse;
 	
     	/* get prototype name */
-    	value = ajTableFetch(t, m->cod);
+    	value = ajTableFetchS(t, m->cod);
     	if(value) 
     	{
     	    ajStrAssignC(&newiso, "");
@@ -1309,6 +1318,9 @@ static void remap_RestrictPreferred(const AjPList l, const AjPTable t)
     return; 
 }
 
+
+
+
 /* @funcstatic remap_Ambiguous  ***************************************
 **
 ** Tests whether there are ambiguity base codes in a string
@@ -1338,6 +1350,9 @@ static AjBool remap_Ambiguous(const AjPStr str)
     return ajFalse;
 }
 
+
+
+
 /* @funcstatic remap_GetFrames **********************************************
 **
 ** Converts the list of frame numbers into a boolean vector.
