@@ -30,7 +30,7 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 				 const AjPStr matchtype, const AjPStr matchtag,
 				 const AjPStr matchvalue,
 				 const AjPStr sortlist,
-				 ajint width, AjBool collapse,
+				 ajint width, AjBool joinfeat, AjBool collapse,
 				 AjBool forward, AjBool reverse,
 				 AjBool unknown, AjBool strand,
 				 AjBool source, AjBool position,
@@ -81,6 +81,7 @@ int main(int argc, char **argv)
     AjPStr matchtag = NULL;
     AjPStr matchvalue = NULL;
     AjPStr sortlist = NULL;
+    AjBool joinfeat;
     AjBool html;
     AjBool id;
     AjBool description;
@@ -108,11 +109,12 @@ int main(int argc, char **argv)
 
     seqall      = ajAcdGetSeqall("sequence");
     outfile     = ajAcdGetOutfile("outfile");
-    matchsource = ajAcdGetString("matchsource");
-    matchtype   = ajAcdGetString("matchtype");
-    matchtag    = ajAcdGetString("matchtag");
-    matchvalue  = ajAcdGetString("matchvalue");
+    matchsource = ajAcdGetString("sourcematch");
+    matchtype   = ajAcdGetString("typematch");
+    matchtag    = ajAcdGetString("tagmatch");
+    matchvalue  = ajAcdGetString("valuematch");
     sortlist    = ajAcdGetListSingle("sort");
+    joinfeat    = ajAcdGetBoolean("joinfeatures");
     html        = ajAcdGetBoolean("html");
     id          = ajAcdGetBoolean("id");
     description = ajAcdGetBoolean("description");
@@ -123,11 +125,11 @@ int main(int argc, char **argv)
     reverse     = ajAcdGetBoolean("reverse");
     unknown     = ajAcdGetBoolean("unknown");
     strand      = ajAcdGetBoolean("strand");
-    source      = ajAcdGetBoolean("source");
+    source      = ajAcdGetBoolean("sourceshow");
     position    = ajAcdGetBoolean("position");
-    type        = ajAcdGetBoolean("type");
-    tags        = ajAcdGetBoolean("tags");
-    values      = ajAcdGetBoolean("values");
+    type        = ajAcdGetBoolean("typeshow");
+    tags        = ajAcdGetBoolean("tagshow");
+    values      = ajAcdGetBoolean("valueshow");
     stricttags  = ajAcdGetBoolean("stricttags");
     annotation  = ajAcdGetRange("annotation"); 
     
@@ -185,7 +187,7 @@ int main(int argc, char **argv)
 	/* show the features */
 	showfeat_ShowFeatSeq(outfile, seq, beg, end, matchsource,
 			     matchtype, matchtag, matchvalue,
-			     sortlist, width, collapse, forward,
+			     sortlist, width, joinfeat, collapse, forward,
 			     reverse, unknown, strand, source,
 			     position, type, tags, values, stricttags,
 			     annotation);
@@ -232,6 +234,7 @@ int main(int argc, char **argv)
 ** @param [r] matchvalue [const AjPStr] tag's value pattern to display
 ** @param [r] sortlist [const AjPStr] type of sorting of features to do
 ** @param [r] width [ajint] width of line of features
+** @param [r] joinfeat [AjBool] join multi-location features on one line
 ** @param [r] collapse [AjBool] show all features on separate lines
 ** @param [r] forward [AjBool] show forward sense features
 ** @param [r] reverse [AjBool] show reverse sense features
@@ -254,7 +257,7 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 				 const AjPStr matchtype, const AjPStr matchtag,
 				 const AjPStr matchvalue,
 				 const AjPStr sortlist,
-				 ajint width, AjBool collapse,
+				 ajint width, AjBool joinfeat, AjBool collapse,
 				 AjBool forward, AjBool reverse,
 				 AjBool unknown, AjBool strand,
 				 AjBool source, AjBool position,
@@ -262,11 +265,17 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 				 AjBool stricttags, const AjPRange annotation)
 {
     AjIList    iter = NULL ;
+    AjIList    itersub = NULL ;
     AjPFeature gf   = NULL ;
-    AjPFeattable feat;
+    AjPFeature gfsub   = NULL ;
+    void **array   = NULL ;
+    AjPFeature *fullfeats   = NULL ;
+    ajuint *featindex = NULL;
+    const AjPFeattable feat;
+    AjPList featlist = NULL;
+    AjPList fullfeatlist = NULL;
     AjPStr lineout;
     char strandout   = '\0';
-    AjBool first     = ajTrue;
     AjPStr sourceout = NULL;
     AjPStr typeout   = NULL;
     AjPStr tagstmp   = NULL;
@@ -280,141 +289,246 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 
     AjBool gotoutput = ajFalse;		 /* have a line to output */
 
-    AjBool want_multiple_line = ajFalse; /* true if want a join()s line */
-    AjBool in_multiple_line = ajFalse;   /* true if this is a join()s line */
-    AjBool join = ajFalse;	         /* want joins on a single line */
-    AjBool child;	                 /* true if multiple's child */
+    AjPStrTok tokens = NULL;
+    AjPStr key = NULL;
+    AjBool val = ajFalse;
+
+    ajuint i;
+    ajuint sizei;
+    ajuint sizej;
+    ajuint j = 0;
 
     /* get the feature table of the sequence */
-    feat = ajSeqGetFeatCopy(seq);
+    feat = ajSeqGetFeat(seq);
     if(!feat)
 	return;
 
-
     lineout = ajStrNew();
     tagstmp = ajStrNewC("");
     tagsout = ajStrNewC("");
     posout  = ajStrNewC("");
 
-    if(ajFeattableSize(feat))
+    if(ajFeattableGetSize(feat))
     {
+        sizei = (ajuint) ajListToarray(feat->Features, (void***) &array);
+        fullfeats = (AjPFeature*) array;
+
+        featlist = ajListNew();
+        AJCNEW0(featindex, sizei);
+
+        sizej = 0;
+        for(i=0;fullfeats[i];i++)
+        {
+            ajListPushAppend(featlist, fullfeats[i]);
+            featindex[sizej++] = i;
+        }
+
 	if(!ajStrCmpC(sortlist, "source"))
 	    /* sort by: sense, source, type, start */
-	    ajListSort(feat->Features, showfeat_CompareFeatSource);
+	    ajListToindex(featlist, featindex, &showfeat_CompareFeatSource);
 	else if(!ajStrCmpC(sortlist, "start"))
-	    /* sort by: sense, start, type, source, source */
-	    ajListSort(feat->Features, showfeat_CompareFeatPos);
+	    /* sort by: sense, start, type, source */
+	    ajListToindex(featlist, featindex, &showfeat_CompareFeatPos);
 	else if(!ajStrCmpC(sortlist, "type"))
 	    /* type */
 	    /* sort by: sense, type, source, start */
-	    ajListSort(feat->Features, showfeat_CompareFeatType);
-	else if(!ajStrCmpC(sortlist, "join"))
-            join = ajTrue;
+	    ajListToindex(featlist, featindex, &showfeat_CompareFeatType);
 	/* else - no sort */
 
-	iter = ajListIterNewread(feat->Features) ;
+	if(!ajStrCmpC(sortlist, "nosort"))
+        {
+            iter = ajListIterNewread(feat->Features);
+        }
+        else 
+        {
+            fullfeatlist = ajListNew();
+            j = 0;
+            for(i=0;i<sizej;i++) 
+            {
+                j = featindex[i];
+                ajListPushAppend(fullfeatlist, fullfeats[j]);
+            }
+            iter = ajListIterNewread(fullfeatlist);
+        }
+        AJFREE(featindex);
 
 	while(!ajListIterDone(iter))
 	{
 	    gf = ajListIterGet(iter);
 
-            /* see if its a child of a multiple join */
-	    child = ajFalse;
-            if(ajFeatIsMultiple(gf))
-	    {
-                if(ajFeatIsChild(gf))
-                    child = ajTrue;
-		else
-		{
-		    /* parent */
-                    want_multiple_line = ajTrue;
-                    in_multiple_line   = ajFalse;
+            ajStrSetClear(&tagstmp);
+	    /* check that we want to output this match of source, type */
+	    if(!showfeat_MatchPatternTags(gf, matchtag, matchvalue,
+					  stricttags, &tagstmp, values))
+                continue;
+            if(!embMiscMatchPatternDelimC(ajFeatGetSource(gf),
+                                          matchsource,",;|"))
+		continue;
+
+            if(ajStrGetLen(matchtype))
+            {
+                val = ajFalse;
+                tokens = ajStrTokenNewC(matchtype, " \t\n\r,;|");
+
+                while (ajStrTokenNextParse( &tokens, &key))
+                {
+                    if (ajFeatTypeMatchWildS(gf, key))
+                    {
+                        val = ajTrue;
+                        break;
+                    }
                 }
-	    }
-	    else
-	    	want_multiple_line = ajFalse;
 
-
-	    /* check that the feature is within the range we wish to display */
+                ajStrTokenDel( &tokens);
+                ajStrDel(&key);
+                if(!val)
+                    continue;
+            }
+            
+/* check that the feature is within the range we wish to display */
 	    if(beg+1 > (ajint)ajFeatGetEnd(gf) || 
 	       end+1 < (ajint)ajFeatGetStart(gf))
 		continue;
 
-            /* ignore remote IDs */
-            if(!ajFeatIsLocal(gf))
-                continue;
+            if(ajListGetLength(gf->Subfeatures))
+            {
+                if(joinfeat)
+                {
+                    if(gotoutput)
+                    {
+                        showfeat_FeatOut(outfile,
+                                         lineout, strandout, sourceout,
+                                         posout, typeout, tagsout,
+                                         width, strand,
+                                         source, type, tags, position);
+                        gotoutput = ajFalse;
+                    }
 
-	    /* check that we want to output this sense */
-	    if(!forward && ajFeatGetStrand(gf) == '+')
-		continue;
-	    if(!reverse && ajFeatGetStrand(gf) == '-')
-		continue;
-	    if(!unknown && ajFeatGetStrand(gf) == '\0')
-		continue;
+                    ajStrSetClear(&lineout);
+                    ajStrAppendCountK(&lineout, ' ', width);
+                    ajStrAssignS(&sourceout, ajFeatGetSource(gf));
+                    ajStrAssignS(&typeout, ajFeatGetType(gf));
+                    strandout = ajFeatGetStrand(gf);
+                    ajStrSetClear(&tagsout);
+                    ajStrSetClear(&posout);
+                }
 
-	    /* check that we want to output this match of source, type */
-	    if(!embMiscMatchPattern(ajFeatGetSource(gf), matchsource) ||
-	       !embMiscMatchPattern(ajFeatGetType(gf), matchtype) ||
-	       !showfeat_MatchPatternTags(gf, matchtag, matchvalue,
-					  stricttags, &tagstmp, values))
-		continue;
+                itersub = ajListIterNewread(gf->Subfeatures);
+                while(!ajListIterDone(itersub))
+                {
+                    gfsub = ajListIterGet(itersub);
+                    /* check that we want to output this sense */
+                    if(!forward && ajFeatGetStrand(gfsub) == '+')
+                        continue;
+                    if(!reverse && ajFeatGetStrand(gfsub) == '-')
+                        continue;
+                    if(!unknown && ajFeatGetStrand(gfsub) == '\0')
+                        continue;
+                    /* ignore remote IDs */
+                    if(!ajFeatIsLocal(gfsub))
+                        continue;
 
-	     /*
-	     ** Starting a new line?
-	     ** Don't start a new line if:
-	     ** collapse and source, type and sense are the same as the last gf
-	     **  or
-	     **  join and child and we are in an existing join multiple line
-	     */
-	    if((!collapse ||
-		first ||
-		ajFeatGetStrand(gf) != strandout ||
-		(source && ajStrCmpCaseS(ajFeatGetSource(gf), sourceout)) ||
-		ajStrCmpCaseS(ajFeatGetType(gf), typeout))
-	       &&
-	       (!join ||
-		! child ||
-		!in_multiple_line))
-	    {
-		if(gotoutput)
-		    showfeat_FeatOut(outfile, lineout, strandout, sourceout,
-				     posout, typeout, tagsout, width, strand,
-				     source, type, tags, position);
+                    if(!joinfeat)
+                    {
+                        if(gotoutput)
+                            showfeat_FeatOut(outfile,
+                                             lineout, strandout, sourceout,
+                                             posout, typeout, tagsout,
+                                             width, strand,
+                                             source, type, tags, position);
 
-		/* reset the strings for the new line */
-		ajStrSetClear(&lineout);
-		ajStrAppendCountK(&lineout, ' ', width);
-		ajStrAssignS(&sourceout, ajFeatGetSource(gf));
-		ajStrAssignS(&typeout, ajFeatGetType(gf));
-		strandout = ajFeatGetStrand(gf);
-		ajStrSetClear(&tagsout);
-		ajStrSetClear(&posout);
+                        /* reset the strings for the new line */
+                        ajStrSetClear(&lineout);
+                        ajStrAppendCountK(&lineout, ' ', width);
+                        ajStrAssignS(&sourceout, ajFeatGetSource(gfsub));
+                        ajStrAssignS(&typeout, ajFeatGetType(gfsub));
+                        strandout = ajFeatGetStrand(gfsub);
+                        ajStrSetClear(&tagsout);
+                        ajStrSetClear(&posout);
 
-		/* something to output */
-		gotoutput = ajTrue;
+                    }
 
-		/* in a multiple line now? */
-		if(want_multiple_line)
-		{
-		    in_multiple_line = ajTrue;
-		    want_multiple_line = ajFalse;
-		}
+                    /* something to output */
+                    gotoutput = ajTrue;
 
-	    }
+                    /* append current tags to tagsout */
+                    ajStrAppendS(&tagsout, tagstmp);
+                    ajStrSetClear(&tagstmp);
 
-	    /* append current tags to tagsout */
-	    ajStrAppendS(&tagsout, tagstmp);
-	    ajStrSetClear(&tagstmp);
+                    /* add positions to posout */
+                    showfeat_AddPos(&posout,
+                                    ajFeatGetStart(gfsub),
+                                    ajFeatGetEnd(gfsub));
 
-	    /* add positions to posout */
-	    showfeat_AddPos(&posout, ajFeatGetStart(gf), ajFeatGetEnd(gf));
+                    /* write the feature on the line */
+                    showfeat_WriteFeat(lineout, strandout,
+                                       ajFeatGetStart(gfsub),
+                                       ajFeatGetEnd(gfsub),
+                                       width, beg, end);
+                }
 
-	    /* write the feature on the line */
-	    showfeat_WriteFeat(lineout, strandout,
-			       ajFeatGetStart(gf), ajFeatGetEnd(gf), width,
-			       beg, end);
+                ajListIterDel(&itersub);
 
-	    first = ajFalse;
+                if(gotoutput)
+                {
+                    showfeat_FeatOut(outfile,
+                                     lineout, strandout, sourceout,
+                                     posout, typeout, tagsout,
+                                     width, strand,
+                                     source, type, tags, position);
+                    gotoutput = ajFalse;
+                }
+            }
+            else
+            {
+                /* check that we want to output this sense */
+                if(!forward && ajFeatGetStrand(gf) == '+')
+                    continue;
+                if(!reverse && ajFeatGetStrand(gf) == '-')
+                    continue;
+                if(!unknown && ajFeatGetStrand(gf) == '\0')
+                    continue;
+
+                /* ignore remote IDs */
+                if(!ajFeatIsLocal(gf))
+                    continue;
+
+		if(!collapse)
+                {
+                    if(gotoutput)
+                        showfeat_FeatOut(outfile,
+                                         lineout, strandout, sourceout,
+                                         posout, typeout, tagsout,
+                                         width, strand,
+                                         source, type, tags, position);
+
+                    /* reset the strings for the new line */
+                    ajStrSetClear(&lineout);
+                    ajStrAppendCountK(&lineout, ' ', width);
+                    ajStrAssignS(&sourceout, ajFeatGetSource(gf));
+                    ajStrAssignS(&typeout, ajFeatGetType(gf));
+                    strandout = ajFeatGetStrand(gf);
+                    ajStrSetClear(&tagsout);
+                    ajStrSetClear(&posout);
+
+                    /* something to output */
+                    gotoutput = ajTrue;
+                }
+
+                /* append current tags to tagsout */
+                ajStrAppendS(&tagsout, tagstmp);
+                ajStrSetClear(&tagstmp);
+
+                /* add positions to posout */
+                showfeat_AddPos(&posout, ajFeatGetStart(gf),
+                                ajFeatGetEnd(gf));
+
+                /* write the feature on the line */
+                showfeat_WriteFeat(lineout, strandout,
+                                   ajFeatGetStart(gf),
+                                   ajFeatGetEnd(gf),
+                                   width, beg, end);
+            }
 	}
 
 	/* print out any last line */
@@ -423,17 +537,15 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 			     typeout, tagsout, width, strand, source, type,
 			     tags, position);
 
-
-
         /* 
         ** Now do the annotation range object, if we have one
         */
-        if (annotation && ajRangeNumber(annotation)) {
+        if (annotation && ajRangeGetSize(annotation)) {
             ann_text = ajStrNew();
             gotoutput = ajFalse;
-            for(count = 0; count < ajRangeNumber(annotation); count++)
+            for(count = 0; count < ajRangeGetSize(annotation); count++)
             {
-                ajRangeValues(annotation, count, &rstart, &rend);
+                ajRangeElementGetValues(annotation, count, &rstart, &rend);
 
 	    /* check that the feature is within the range we wish to
                display */
@@ -441,7 +553,7 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 		    continue;
 
                 /* get the annotation text */                 
-                ajRangeText(annotation, count, &ann_text);
+                ajRangeElementGetText(annotation, count, &ann_text);
 
            /* don't start a new line if collapse and previous text is
               the same */
@@ -485,15 +597,15 @@ static void showfeat_ShowFeatSeq(AjPFile outfile, const AjPSeq seq, ajint beg,
 
     }
 
-
-    ajFeattableDel(&feat);
-
     ajStrDel(&tagstmp);
     ajStrDel(&tagsout);
     ajStrDel(&posout);
     ajStrDel(&lineout);
     ajStrDel(&sourceout);
     ajStrDel(&typeout);
+    ajListFree(&featlist);
+    ajListFree(&fullfeatlist);
+    AJFREE(array);
 
     return;
 }
@@ -850,7 +962,7 @@ static AjBool showfeat_MatchPatternTags(const AjPFeature feat,
     titer = ajFeatTagIter(feat);
     while(ajFeatTagval(titer, &tagnam, &tagval))
     {
-        tval = embMiscMatchPattern(tagnam, tpattern);
+        tval = embMiscMatchPatternDelimC(tagnam, tpattern,",;|");
 
          /*
 	 ** If tag has no value then
@@ -876,7 +988,7 @@ static AjBool showfeat_MatchPatternTags(const AjPFeature feat,
 	     ** without spitting it up into words. 
 	     */
             vval = (ajStrMatchS(tagval, vpattern) ||
-		    embMiscMatchPattern(tagval, vpattern));
+		    embMiscMatchPatternDelimC(tagval, vpattern,",;|"));
 	}
 
         if(tval && vval)
