@@ -4,9 +4,9 @@
 # DONE
 ##########################
 # Timeout (TI line to set) default 60secs, diffseq needs more
-# Preprocess (PP line) command
-# Postprocess (QQ line) command - e.g. testing database builds, reusing output
-# EMBOSS_RC variable to read and extra .embossrc file (for new test dbs)
+# Pre-process (PP line) command
+# Post-process (QQ line) command - e.g. testing database builds, reusing output
+# EMBOSS_RC variable to read an extra .embossrc file (for new test databases)
 #
 ##########################
 # THINGS TO DO
@@ -24,15 +24,15 @@
 # ER error return code
 # ## Comment (required double #)
 # CC Comment (used in commenting on failed tests, e.g. "Requires SRS"
-# DL success or all or keep - whether to delete the files afterways
+# DL success or all or keep - whether to delete the files afterwards
 # IN Line(s) of standard input
 # FI File name (stdout and stderr assumed to exist and be empty unless stated)
 # FK Keystrokes output File name (non-text)
 # FP File pattern - /regexp/ to be found. Optional count to check exact number.
 # FZ [<=>]number File size test. Implicit test for zero size stdout/stderr
 #                unless stated
-# FC [<=>]number File linecount test
-# UC Comment (used un documenting the usage)
+# FC [<=>]number File line count test
+# UC Comment (used in documenting the usage)
 # IC Comment (used in documenting the input files)
 # OC Comment (used in documenting the output files)
 # RQ Requires (e.g. SRS for tests that need a local getz working)
@@ -52,10 +52,76 @@
 
 sub usage () {
   print STDERR "Usage:\n";
-  print STDERR "  qatest.pl [-kk | -ks | -ka] [-t=60] [-wild] [-mcheck] [testnames...]\n";
+  print STDERR "  qatest.pl [-kk | -ks | -ka] [-t=60] [-wild] [-mcheck] [-noembassy] [-embassy=package] [testnames...]\n";
   print STDERR "            defaults: -kk -t=60\n";
 }
 
+###################################################################
+# qatestsystem
+#
+# system call with pre and post processing
+# run as-is on Unix/cygwin
+# converted for windows
+###################################################################
+
+sub qatestsystem($$$){
+  my ($pp, $cmd, $qq) = @_;
+  my $sysstat;
+  if(!$iswindows) {
+    $sysstat = system("$pp $cmd $qq");
+    return $sysstat;
+  }
+#  print "Pre  '$pp'\n";
+#  print "Cmd  '$cmd'\n";
+#  print "Post '$qq'\n";
+
+  open (BAT, ">qatest.bat") || die "Cannot open qatest.bat";
+  print BAT "\@echo off\n";
+
+  my $c;
+  my $i=0;
+
+  foreach $c (split(/;/, $pp)){
+    $c =~ s/^ +//;
+    $c =~ s/ +$//;
+    $c =~ s/\//\\/g;
+    $c =~ s/^cp /copy /;
+    $c =~ s/^rm /del \/q /;
+    $c =~ s/^(\S+)=/SET $1=/;
+    if($c eq "") {next}
+    if($c =~ /export /i) {next}
+    if($iswindows) {
+      $c =~ s/~(\S+)/\"\%HOMEDRIVE\%\%HOMEPATH\%$1\"/;
+    }
+    $i++;
+#    print "    Pre[$i] '$c'\n";
+    print BAT "$c\n";
+  }
+  print BAT "$cmd\n";
+
+  foreach $c (split(/;/, $qq)){
+    $c =~ s/^ +//;
+    $c =~ s/ +$//;
+    $c =~ s/\//\\/g;
+    $c =~ s/^rm -rf/rmdir \/s \/q /;
+    $c =~ s/^rm /del \/q /;
+    $c =~ s/^cp /copy /;
+    if($c eq "") {next}
+    if($c =~ /export /i) {next}
+    if($iswindows) {
+      $c =~ s/~(\S+)/\"\%HOMEDRIVE\%\%HOMEPATH\%$1\"/;
+    }
+    $i++;
+#    print "   Post[$i] '$c'\n";
+    print BAT "$c\n";
+  }
+  close BAT;
+
+  $sysstat = system("qatest.bat");
+
+  return $sysstat;
+}
+
 ###################################################################
 # runtest
 #
@@ -80,7 +146,8 @@ sub runtest ($) {
   my $testid = "";
   my $testin = "";
   my $timeout = $timeoutdef;
-  my $ppcmd = "EMBOSSRC=../../ ;export EMBOSSRC ;EMBOSS_RCHOME=N ;export EMBOSS_RCHOME ;";
+
+  my $ppcmd = "$ppcmd";
   my $qqcmd = "";
   my %testfile = ();
   my %outfile = ();
@@ -97,6 +164,13 @@ sub runtest ($) {
   $globaltestdelete=$defdelete;	# global cleanup of test directory
   $globalcomment = ""; # global comment in case of expected failure
 
+  if($iswindows) {
+      if($dodebug) {$ppcmd .= "SET EMBOSS_DEBUG=Y;"}
+  }
+  else {
+      if($dodebug) {$ppcmd .= "EMBOSS_DEBUG=Y ;export EMBOSS_DEBUG ;"}
+  }
+
 # parse the test definition (EMBL-style 2 character prefixes)
 
   foreach $line  (split (/^/, $testdef)) {
@@ -113,17 +187,27 @@ sub runtest ($) {
       $ifile=0;
       $packa = "unknown";
       print LOG "Test <$testid>\n";
-      $sysstat = system( "rm -rf $testid");
-      $status = $sysstat >> 8;
-      if ($status) {
-	$testerr = "failed to delete old directory $testid, status $status\n";
-	print STDERR $testerr;
-	print LOG $testerr;
+      if(!$repeattest){
+#	  print "+++ clean up old '$id' directory\n";
+	  if(-e $id && -d $testid) {
+#	  print "+++ calling rmdir\n";
+	  if($iswindows){
+	      $sysstat = system( "rmdir /s /q $testid");
+	  } else {
+	      $sysstat = system( "rm -rf $testid");
+	  }
+	  $status = $sysstat >> 8;
+	  if ($status) {
+	      $testerr = "failed to delete old directory $testid, status $status\n";
+	      print STDERR $testerr;
+	      print LOG $testerr;
+	  }
+	  }
+	  mkdir ("$testid", 0777);
+	  open (SAVEDEF, ">$1/testdef") || die "Cannot save $1/testdef";
+	  print SAVEDEF $testdef;
+	  close (SAVEDEF);
       }
-      mkdir ("$testid", 0777);
-      open (SAVEDEF, ">$1/testdef") || die "Cannot save $1/testdef";
-      print SAVEDEF $testdef;
-      close (SAVEDEF);
     }
 
 # other lines
@@ -141,14 +225,25 @@ sub runtest ($) {
     elsif ($line =~ /^AP\s+(\S+)/) {
 	$testapp = $1;
 	$apcount{$testapp}++;
-	if (!defined($tfm{$testapp})) {
+	if (!$simple && !defined($tfm{$testapp})) {
 	    $tfm{$testapp}=0;
-	    $dtop = "../../doc/programs/";
-	    if (-e "$dtop/html/$testapp.html") {$tfm{$testapp}++}
+	    if($iswindows) {
+	      $dtop = "$basedir\\doc\\programs";
+	      $dochtml = "$dtop\\html\\$testapp.html";
+	      $doctext = "$dtop\\text\\$testapp.txt";
+	      $docmain = "$dtop\\master\\emboss\\apps\\$testapp.html";
+	    }
+	    else {
+	      $dtop = "$basedir/doc/programs";
+	      $dochtml = "$dtop/html/$testapp.html";
+	      $doctext = "$dtop/text/$testapp.txt";
+	      $docmain = "$dtop/master/emboss/apps/$testapp.html";
+	    }
+	    if (-e "$dochtml") {$tfm{$testapp}++}
 	    else {print STDERR "No HTML docs for $testapp\n";$misshtml++;}
-	    if (-e "$dtop/text/$testapp.txt") {$tfm{$testapp}++}
+	    if (-e "$doctext") {$tfm{$testapp}++}
 	    else {print STDERR "No tfm text docs for $testapp\n";$misstext++;}
-	    if (-e "$dtop/master/emboss/apps/$testapp.html") {$sf{$testapp}++}
+	    if (-e "$docmain") {$sf{$testapp}++}
 	    else {print STDERR "No master (sourceforge) docs for $testapp\n";$misssf++;}
 	}
     }
@@ -163,6 +258,13 @@ sub runtest ($) {
 	$apcount{$testapp}++;
 	### no need to test docs for a make check application
     }
+    elsif ($line =~ /^AX\s*(.*)/) {
+	$testa = 1;
+	$testapp = $1;
+	if ($packa eq "unknown") {
+	    print STDERR "No AY line before AX line in test $testid\n";
+	}
+    }
     elsif ($line =~ /^AA\s*(.*)/) {
 	$testa = 1;
 	$testapp = $1;
@@ -171,7 +273,7 @@ sub runtest ($) {
 	    print STDERR "No AB line before AA line in test $testid\n";
 	}
 	$dtop = "../../embassy/$packa/emboss_doc";
-	if (!defined($tfm{$testapp})) {
+	if ($doembassy && !defined($tfm{$testapp})) {
 	    $tfm{$testapp}=0;
 	    if (-e "$dtop/html/$testapp.html") {$tfm{$testapp}++}
 	    else {print STDERR "No HTML docs for $testapp\n";$misshtml++;}
@@ -182,9 +284,19 @@ sub runtest ($) {
 	}
     }
     elsif ($line =~ /^AB\s*(.*)/) {$packa = $1}
+    elsif ($line =~ /^AY\s*(.*)/) {$packa = $1}
     elsif ($line =~ /^CL\s+(.*)/) {
-      if ($cmdline ne "") {$cmdline .= " "}
-      $cmdline .= $1;
+        $newcmdline = $1;
+	if($iswindows) {
+	  $newcmdline =~ s/~(\S+)/\"\%HOMEDRIVE\%\%HOMEPATH\%$1\"/;
+	}
+	if ($cmdline ne "") {
+	    if($iswindows && ($cmdline =~ /\\$/)){
+	        $cmdline =~ s/\\$//g;
+	    }
+	    else {$cmdline .= " "}
+        }
+        $cmdline .= $newcmdline;
     }
 
 # directoryname - must be unique
@@ -230,8 +342,10 @@ sub runtest ($) {
 
 # filename - must be unique
 
-    elsif ($line =~ /^F[IK]\s+(\S+)/) {
+    elsif ($line =~ /^F[IK]\s+(\S+)(\s+binary)?/) {
       $filename = $1;
+      $isbinary = $2;
+      if(!defined($isbinary)){$isbinary = 0}
       if (defined($outfile{$filename})) {
 	$testerr = "$retcode{16} $testid/$filename\n";
 	print STDERR $testerr;
@@ -240,9 +354,10 @@ sub runtest ($) {
       }
       $outfile{$filename} = $ifile;
       $outfilepatt{$ifile} = $ipatt;
-      print LOG "Known file [$ifile] <$1>\n";
+      print LOG "Known file [$ifile] <$filename>\n";
       $ifile++;
       $filezero{$ifile} = 0;
+      $binfile{$ifile-1} = $isbinary;
     }
 
 # file pattern(s) - can be many for each file
@@ -302,27 +417,26 @@ sub runtest ($) {
   }
 
   if ($testq) {	# for "make check" apps (AQ lines) we can skip
-    if($packa eq "") {
+    if($packa eq "unknown") {
       $testpath = "../../emboss/"; #  up from the test/qa directory
     }
     else {
-      $testpath = "../../embassy/$packa/source/"; #  up from the test/qa directory
+      $testpath = "../../embassy/$packa/src/"; #  up from the test/qa directory
     }
-    if (! (-e "$testpath$testapp")) {$skipcheck++; return 0} # make check not run
+    if (! (-e "$testpath$testapp")) {$skipcheck++; print "not found '$testpath$testapp'\n";return 0} # make check not run
     if ($testappname && defined($acdname{$testapp}) && $acdname{$testapp}) {
       print STDERR "Check application $testapp installed - possible old version\n";
     }
     $testpath = "../$testpath";	# we run from the test/qa/* subdirectory
   }
 
-  if ($testa) {	# for "embassy" apps (AA lines) we can skip
+  if ($doembassy && $testa) {	# for "embassy" apps (AA lines) we can skip
     if ($testappname && !defined($acdname{$testapp})) { # embassy make not run
 	print STDERR "Embassy application $testapp ($packa) not installed - skip\n";
-      $skipembassy++;
-      return 0;
+	$globaltestdelete = "all";
+	$skipembassy++;
+	return 0;
     }
-
-#    if ($testappname && !defined($acdname{$testapp})) {$skipembassy++; return 0} # embassy make not run
   }
 
 # cd to the test directory (created when ID was parsed)
@@ -332,87 +446,96 @@ sub runtest ($) {
 # set up stdin always
 # we need to hit EOF if it tries to read when there is no input
 
-  open (TESTIN, ">stdin");
-  if ($testin ne "") {
-    print TESTIN $testin;
-  }
-  close TESTIN;
-  $stdin = "< stdin";
-
-  $timealarm=0;
   $starttime = time();
+  
+  if($repeattest){
+      system("rm testlog");
+  }
+  if(!$repeattest){
+      open (TESTIN, ">stdin");
+      if ($testin ne "") {
+	  print TESTIN $testin;
+      }
+      close TESTIN;
+      $stdin = "< stdin";
+
+      $timealarm=0;
 
 # run the test with a timeout (default 60 seconds) to catch infinite loops
 # The easiest infinite loop is an unexpected prompt, which waits on stdin
 
-  eval {
-    $status = 0;
-    alarm($timeout);
-    $sysstat = system("$domcheck$ppcmd $testpath$testapp $cmdline > stdout 2> stderr $stdin $qqcmd");
-    alarm(0);
-    $status = $sysstat >> 8;
-  };
+      eval {
+	  $status = 0;
+	  alarm($timeout);
+          $cmdstr = "$testpath$testapp $cmdline > stdout 2> stderr $stdin";
+#          print STDERR "$cmdstr\n";
+	  $sysstat = qatestsystem("$domcheck$ppcmd", "$cmdstr", "$qqcmd");
+	  alarm(0);
+	  $status = $sysstat >> 8;
+      };
 
 
-  if ($@) {			# error from eval block
-    if ($@ =~ /qatest timeout/) {	# timeout signal handler
-      $timealarm = 1;		# set timeout flag and continue
-    }
-    else {			# other signal - fail
-      die;
-    }
-  }
+      if ($@) {			# error from eval block
+	  if ($@ =~ /qatest timeout/) {	# timeout signal handler
+	      $timealarm = 1;		# set timeout flag and continue
+	  }
+	  else {			# other signal - fail
+	      die;
+	  }
+      }
 
 # Report any timeout
 
-  if ($timealarm) {
-    $testerr = "$retcode{11} ($timeout secs) '$testapp $cmdline $stdin', status $status/$testret\n";
-    print STDERR $testerr;
-    print LOG $testerr;
-    chdir ("..");
-    return 11;
-  }
+      if ($timealarm) {
+	  $testerr = "$retcode{11} ($timeout secs) '$testapp $cmdline $stdin', status $status/$testret\n";
+	  print STDERR $testerr;
+	  print LOG $testerr;
+	  chdir ("..");
+	  return 11;
+      }
 
 # report any failed system call
 
-  elsif ($status) {
-    if ($status != $testret) {
-      $testerr = "$retcode{2} '$testapp $cmdline $stdin', status $status/$testret\n";
-      print STDERR $testerr;
-      print LOG $testerr;
-      chdir ("..");
-      return 2;
-    }
-  }
+      elsif ($status) {
+	  if ($status != $testret) {
+	      $testerr = "$retcode{2} '$testapp $cmdline $stdin', status $status/$testret\n";
+	      print STDERR $testerr;
+	      print LOG $testerr;
+	      chdir ("..");
+	      return 2;
+	  }
+      }
 
 # report any run that succeeded where a failure was expected
 
-  else {
-    if ($testret) {
-      $testerr = "$retcode{5} '$testapp $cmdline $stdin', status $status/$testret\n";
-      print STDERR $testerr;
-      print LOG $testerr;
-      chdir ("..");
-      return 5;
-    }
-  }
+      else {
+	  if ($testret) {
+	      $testerr = "$retcode{5} '$testapp $cmdline $stdin', status $status/$testret\n";
+	      print STDERR $testerr;
+	      print LOG $testerr;
+	      chdir ("..");
+	      return 5;
+	  }
+      }
 
 # We have a successful run, no timeouts.
 # Check it did what we wanted it to
 
 # Note the run time
 
+  }
+
   $endtime = time();
   $runtime = $endtime - $starttime;
 
 # Check for a core dump
 
   if (-e "core") {
-    $testerr = "$retcode{12} $testid\n";
-    print STDERR $testerr;
-    print LOG $testerr;
-    chdir ("..");
-    return 12;
+      $testerr = "$retcode{12} $testid\n";
+      print STDERR $testerr;
+      print LOG $testerr;
+      chdir ("..");
+      return 12;
   }
 
 # Read the file names in the test directory
@@ -503,7 +626,9 @@ sub runtest ($) {
 # This file was not defined (we let extra -debug files through)
 
     if (!defined($outfile{$file})) {
+      if($iswindows && ($file eq "qatest.bat")) {next}
       if($file eq "$testapp.dbg") {next}
+      if($file eq "$testapp.ax2log") {next}
       $testerr = "$retcode{3} $testid/$file\n";
       print STDERR $testerr;
       print LOG $testerr;
@@ -517,10 +642,20 @@ sub runtest ($) {
       $i =  $outfile{$file};
       print LOG "file [$i] <$file>\n";
 
+# read the output file - to test patterns and line count
+
+      open (OFIL, $file) || die "Cannot open $testid/$file";
+      $odata = "";
+      $linecount=0;
+      while (<OFIL>) {$odata .= $_; $linecount ++;}
+      close OFIL;
+
 # File size defined - test it
 
       if (defined($outsize{$i})) {
 	$size = -s $file;
+	if($iswindows && !$binfile{$i} &&
+	   ($file eq "stderr" || $file eq "stdout")) {$size -= $linecount}
 	print LOG "Test size $size '$outsize{$i}' $testid/$file\n";
 	if (testnum($outsize{$i}, $size)) {
 	  $testerr = "$retcode{7} $size '$outsize{$i}' $testid/$file\n";
@@ -531,18 +666,17 @@ sub runtest ($) {
 	}
       }
 
-# read the output file - to test patterns and line count
-
-      open (OFIL, $file) || die "Cannot open $testid/$file";
-      $odata = "";
-      $linecount=0;
-      while (<OFIL>) {$odata .= $_; $linecount ++;}
-      close OFIL;
-
 # Test line count (FC)
 
       if (defined($outcount{$i})) {
 	print LOG "Test linecount $linecount '$outcount{$i}' $testid/$file\n";
+	if ($binfile{$i}) {
+	  $testerr = "$retcode{8} '$outcount{$i}' BINARY FILE $testid/$file\n";
+	  print STDERR $testerr;
+	  print LOG $testerr;
+	  chdir ("..");
+	  return 8;
+	}
 	if (testnum($outcount{$i}, $linecount)) {
 	  $testerr = "$retcode{8} $linecount '$outcount{$i}' $testid/$file\n";
 	  print STDERR $testerr;
@@ -735,7 +869,7 @@ $testappname=0;
 $misshtml=0;
 $misstext=0;
 $misssf=0;
-%without = ();
+%without = ("srs" => 1);
 %dotest = ();
 %tfm = ();
 %sf = ();
@@ -745,12 +879,51 @@ $packa="unknown";
 $dowild=0;
 $logfile = "qatest.log";
 $testwild = "*";
+$dodebug=0;
+$doembassy=1;
+$docheck=1;
+$repeattest=0;
+
+open (VERSION, "embossversion -full -filter|") ||
+    die "Cannot run embossversion";
+while (<VERSION>){
+    if(/System: (\S+)/) {
+      $os = $1;
+      if($os =~ /windows/) {$iswindows = 1; $docheck=0; $doembassy=0}
+    }
+    if(/BaseDirectory: (\S+)/) {
+	$basedir = $1;
+#        print "BaseDirectory: $acddir\n";
+    }
+}
+close VERSION;
+
+if($iswindows){
+  $ppcmd = "SET EMBOSSRC=..\\..\\;SET EMBOSS_RCHOME=N;";
+  $qatestfile = "$basedir\\test\\qatest.dat";
+  $acddir = "$basedir\\acd";
+  chdir "$basedir\\test\\qa";
+}
+else {
+  $ppcmd = "EMBOSSRC=../../ ;export EMBOSSRC ;EMBOSS_RCHOME=N ;export EMBOSS_RCHOME ;";
+  $qatestfile = "$basedir/test/qatest.dat";
+  $acddir=  "$basedir/emboss/acd";
+  chdir "$basedir/test/qa";
+}
+$simple=0;
 
 foreach $test (@ARGV) {
   if ($test =~ /^-(.*)/) {
     $opt=$1;
     if ($opt eq "kk") {$defdelete="keep"}
+    elsif ($opt eq "keep") {$defdelete="keep"}
     elsif ($opt eq "ks") {$defdelete="success"}
+    elsif ($opt eq "ka") {$defdelete="all"}
+    elsif ($opt eq "debug") {$dodebug=1}
+    elsif ($opt eq "noembassy") {$doembassy=0}
+    elsif ($opt eq "nocheck") {$docheck=0}
+    elsif ($opt eq "retest") {$repeattest=1}
+    elsif ($opt eq "simple") {$simple=1}
     elsif ($opt eq "wild") {
 	$dowild=1;
 	if(defined($testname)) {
@@ -758,10 +931,13 @@ foreach $test (@ARGV) {
 	}
     }
     elsif ($opt eq "mcheck") {$domcheck="MALLOC_CHECK_=3;export MALLOC_CHECK_;"}
-    elsif ($opt eq "ka") {$defdelete="all"}
     elsif ($opt =~ /without=(\S+)/) {$without{$1}=1}
+    elsif ($opt =~ /with=(\S+)/) {undef($without{$1})}
     elsif ($opt =~ /t=([0-9]+)/) {$timeoutdef=int($1)}
     elsif ($opt =~ /logfile=(\S+)/) {$logfile=">$1"} # append to logfile
+    elsif ($opt =~ /testfile=(\S+)/) {$qatestfile="$1"}
+    elsif ($opt =~ /acd=(\S+)/) {$acddir="$1"}
+    elsif ($opt =~ /embassy=(\S+)/) {$embassyonly="$1"}
     else {print STDERR "+++ unknown option '$opt'\n"; usage()}
   }
   else {
@@ -826,7 +1002,7 @@ $SIG{ALRM} = sub { print STDERR "+++ timeout handler\n"; die "qatest timeout" };
 # The relative path is fixed, as are the paths of files in the qatest.dat
 # file, so best to keep everything running in the test/qa directory
 
-opendir (ACDDIR, "../../emboss/acd") || die "Cannot open emboss/acd directory";
+opendir (ACDDIR, "$acddir") || die "Cannot open ACD directory '$acddir'";
 @acdfiles = readdir(ACDDIR);
 closedir ACDDIR;
 
@@ -840,7 +1016,13 @@ if (!$numtests) {
 
   undef @acdfiles;
 
-  open (WOSSNAME, "export -n EMBOSS_ACDCOMMANDLINE;wossname -alpha -auto|") || die "Cannot run wossname";
+  if($iswindows) {
+      open (WOSSNAME, "wossname -alpha -auto|") || die "Cannot run wossname";
+  }
+  else {
+      open (WOSSNAME, "unset EMBOSS_ACDCOMMANDLINE;wossname -alpha -auto|") ||
+	  die "Cannot run wossname";
+  }
   while (<WOSSNAME>) {
     if (/^[a-z]\S+/) {
       $app = $&;
@@ -856,12 +1038,31 @@ if (!$numtests) {
 
 }
 
-open (IN, "../qatest.dat") || die "Cannot open qatest.dat";
+@testfiles = ("$qatestfile");
+if($doembassy) {
+    opendir (EMBASSY, "$basedir/embassy") ||
+	die "Failed to find embassy directory";
+    while ($edir = readdir(EMBASSY)) {
+	if ($edir =~ /^[.]/) {next}
+	if(defined($embassyonly) && ($embassyonly ne $edir)) {next}
+	if(-d "$basedir/embassy/$edir") {
+	    if(-e "$basedir/embassy/$edir/test/qatest.dat") {
+		push @testfiles, "$basedir/embassy/$edir/test/qatest.dat";
+	    }
+	}
+    }
+}
+
+foreach $qatestfile (@testfiles) {
+open (IN, "$qatestfile") || die "Cannot open $qatestfile";
 open (LOG, ">$logfile") || die "Cannot open $logfile";
 
 # make qatest.log unbuffered and be sure to reset the current filehandle
 $fh = select LOG; $|=1; select $fh;
 
+$ischeck = 0;
+$isembassy = 0;
+
 while (<IN>) {
 
 # Save a test when we reach an ID line (ignore any comments in between tests)
@@ -870,7 +1071,17 @@ while (<IN>) {
     $lastid = $id;
     $id = $1;
     $testdef = "";
+    $ischeck = 0;
+    $isembassy = 0;
   }
+  if (/^AA\s+(\S+)/) {$isembassy = 1}
+  if (/^AX\s+(\S+)/) {$isembassy = 1}
+  if (/^AB\s+(\S+)/) {$embassypack = $1}
+  if (/^AY\s+(\S+)/) {
+      if(defined($embassyonly)) {$embassypack = $embassyonly}
+      else {$embassypack = $1}
+  }
+  if (/^AQ\s+/) {$ischeck = 1}
   $testdef .= $_;
 
 # end of definition - fire up the test
@@ -878,7 +1089,12 @@ while (<IN>) {
   if (/^\/\//) {
     if (($numtests > 0) && !$dowild && !$dotest{$id}) {next}
     if (($numtests > 0) && $dowild && $id !~ /$testwild/) {next}
-
+    if($isembassy){
+      if(!$doembassy) {next}
+      if(defined($embassyonly) && ($embassyonly ne $embassypack)){$testappname=0;next}
+    }
+    elsif(defined($embassyonly)){$testappname=0;next}
+    if($ischeck && !$docheck) {next}
     $result = runtest ($testdef);
     $tcount++;
 
@@ -916,13 +1132,23 @@ while (<IN>) {
 # Usually we keep failed tests (unless delete is set to 'all')
 
       if ($globaltestdelete eq "all") {
-	$sysstat = system( "rm -rf $id");
-	$status = $sysstat >> 8;
-	if ($status) {
-	  $testerr = "failed to delete old directory $id, status $status\n";
-	  print STDERR $testerr;
-	  print LOG $testerr;
-	}
+	  if(!$repeattest){
+#              print "+++ clean up old '$id' directory\n";
+              if(-e $id && -d $id) {
+#              print "+++ calling rmdir\n";
+              if($iswindows){
+	        $sysstat = system( "rmdir /s /q $id");
+              } else {
+	        $sysstat = system( "rm -rf $id");
+              }
+	      $status = $sysstat >> 8;
+	      if ($status) {
+		  $testerr = "failed to delete old directory $id, status $status\n";
+		  print STDERR $testerr;
+		  print LOG $testerr;
+	      }
+              }
+	  }
       }
 
     }
@@ -938,12 +1164,19 @@ while (<IN>) {
 # usually we delete successful results (unless delete is set to 'keep')
 
       if ($globaltestdelete ne "keep") {
-	$sysstat = system( "rm -rf $id");
-	$status = $sysstat >> 8;
-	if ($status) {
-	  $testerr = "failed to delete old directory $id, status $status\n";
-	  print STDERR $testerr;
-	  print LOG $testerr;
+	if(!$repeattest){
+	  if($iswindows) {
+	    $sysstat = system( "rmdir /s /q $id");
+	  }
+	  else {
+	    $sysstat = system( "rm -rf $id");
+	  }
+	    $status = $sysstat >> 8;
+	    if ($status) {
+		$testerr = "failed to delete old directory $id, status $status\n";
+		print STDERR $testerr;
+		print LOG $testerr;
+	    }
 	}
       }
 
@@ -953,7 +1186,7 @@ while (<IN>) {
 
     print LOG "\n";
 
-# clear any global hashes befoire defining the next test
+# clear any global hashes before defining the next test
 
     undef %outfilepatt;
     undef %pattest;
@@ -965,7 +1198,7 @@ while (<IN>) {
 
   }
 }
-
+}
 # Final summary
 
 if ($testappname) {
@@ -994,10 +1227,12 @@ foreach $x (sort(keys(%packfail))) {
     }
 }
 print STDERR "Tests total: $totall pass: $tpass fail: $tfail\n";
-print STDERR "Skipped: $totskip check: $skipcheck embassy: $skipembassy requirements: $skipreq\n";
+if(!$simple) {
+    print STDERR "Skipped: $totskip check: $skipcheck embassy: $skipembassy requirements: $skipreq\n";
 
-print STDERR "No tests: $tnotest\n";
-print STDERR "Missing documentation html: $misshtml text: $misstext sourceforge: $misssf\n";
+    print STDERR "No tests: $tnotest\n";
+    print STDERR "Missing documentation html: $misshtml text: $misstext sourceforge: $misssf\n";
+}
 print STDERR "Time: $alltime seconds\n";
 print LOG "Time: $alltime seconds\n";
 
